#include <iostream>
#include "String.h"

using namespace std;

template <class T>
class Element {
public:
	void show();

	T *data;
	Element *next;
};

template <class T>
class Dynamo {
public:
	Dynamo();
	~Dynamo();

	void add(T data);
	bool remove(Element<T> *elem);
	void edit(Element<T> *elem, T data);
	void show();

	Element<T> *get(const wchar_t *str);
	Element<T> *get(const int value);

	void sort();
	void sortByName();

private:
	Element<T> *head;
};

template <class T>
Dynamo<T>::Dynamo()
{
	head = NULL;
}

template <class T>
Dynamo<T>::~Dynamo()
{
	while (head != NULL) {
		Element<T> *temp = head->next;
		delete head;
		head = temp;
	}
}

template <class T>
void Dynamo<T>::add(T data)
{
	Element<T> *temp = new Element<T>;
	temp->data = new T(data);
	temp->next = head;
	head = temp;
}

template <class T>
bool Dynamo<T>::remove(Element<T> *elem)
{
	Element<T> *temp = head;

	if (temp == elem) {
		head = elem->next;
		return true;
	}

	while (temp != NULL) {
		if (temp->next == elem) {
			temp->next = elem->next;
			delete elem;
			return true;
		}
		temp = temp->next;
	}
	return false;
}

template <class T>
void Dynamo<T>::edit(Element<T> *elem, T data)
{
	Element<T> *temp = new Element<T>;
	temp->data = new T(data);
	temp->next = elem->next;
	*elem = *temp;
}

template <class T>
void Dynamo<T>::show()
{
	Element<T> *temp = head;

	while (temp != NULL) {
		wcout << *temp->data << endl;
		temp = temp->next;
	}
}

template <class T>
Element<T> *Dynamo<T>::get(const int value)
{
	Element<T> *temp = head;

	while (temp != NULL) {
		if (temp->data == value) {
			return temp;
		}
		temp = temp->next;
	}
	return NULL;
}

template <class T>
Element<T> *Dynamo<T>::get(const wchar_t *str)
{
	Element<T> *temp = head;
	String buf_str(str);

	while (temp != NULL) {
		if (temp->data == str) {
			return temp;
		}
		temp = temp->next;
	}
	return NULL;
}

template <class T>
void Dynamo<T>::sort()
{
	if (head) {
		Element<T> *res = head;
		head = head->next;
		res->next = NULL;

		while (head) {
			Element<T> *elem = head;
			head = head->next;

			if (*elem->data < *res->data) {
				elem->next = res;
				res = elem;
			} else {
				Element<T> *p = res;

				while (p->next) {
					if (*p->next->data > *elem->data) {
						break;
					}

					p = p->next;
				}

				elem->next = p->next;
				p->next = elem;
			}
		}
		head = res;
	}
}